﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

/// Задачката, която решавахме на семинара
namespace PerformanceReviews_Visitor
{

    # region Reviewables

    public interface IReviewable
    {
        IReviewer Manager { get; set; }
        void InitiateReview();
    }

    public abstract class ReviewableBase : IReviewable
    {
        public IReviewer Manager { get; set; }
        public abstract void InitiateReview();
    }

    public class Developer : ReviewableBase
    {
        //... Dev specific logic here

        public override void InitiateReview()
        {
            Manager.Review(this);
        }

        public override string ToString()
        {
            return "George the Developer";
        }
    }

    public class QAEngineer : ReviewableBase
    {
        //... QA specific logic here

        public override void InitiateReview()
        {
            Manager.Review(this);
        }

        public override string ToString()
        {
            return "John the QA";
        }
    }

    #endregion

    #region Reviewers

    public interface IReviewer
    {
        void Review(Developer developer);
        void Review(QAEngineer qaEngineer);
    }

    public class TechnicalLead : IReviewer
    {
        public void Review(Developer developer)
        {
            Console.WriteLine("The TL is interviewing a dev: " + developer);
        }

        public void Review(QAEngineer qaEngineer)
        {
            Console.WriteLine("The TL is interviewing a QA: " + qaEngineer);
        }
    }

    public class ChiefTechnicalOfficer : IReviewer
    {
        public void Review(Developer developer)
        {
            Console.WriteLine("The CTO is interviewing a dev: " + developer);
        }

        public void Review(QAEngineer qaEngineer)
        {
            Console.WriteLine("The CTO is interviewing a QA: " + qaEngineer);
        }
    }

    #endregion

    class Program
    {
        static void Main(string[] args)
        {
            var cto = new ChiefTechnicalOfficer();
            var tl = new TechnicalLead();

            var dev = new Developer
            {
                Manager = cto
            };

            var qa = new QAEngineer
            {
                Manager = tl
            };

            dev.InitiateReview();
            qa.InitiateReview();
        }
    }
}
